class PgMaterializedView

  def self.db
    ActiveRecord::Base.connection
  end

  def self.instances
    @instances ||= Hash.new { |h, name| h[name] = new(name) }
  end

  def self.names
    db.select_values("SELECT oid::regclass::text FROM pg_class WHERE relkind = 'm'")
  end

  def self.[](name)
    name = name.to_s
    raise "not a materialized view: #{name}" unless names.include?(name)
    instances[name]
  end

  def self.all
    names.map { |name| instances[name] }
  end

  def self.refresh_all
    all.each(&:refresh)
  end

  attr_reader :name

  def initialize(name)
    @name = name
  end

  # Public: refresh the materialized view, populates the view if needed
  def refresh
    begin
      self.class.db.execute("REFRESH MATERIALIZED VIEW CONCURRENTLY #{name}")
    rescue ActiveRecord::StatementInvalid => e
      raise e unless e.message =~ /CONCURRENTLY cannot be used when the materialized view is not populated/
      self.class.db.execute("REFRESH MATERIALIZED VIEW #{name}")
    end
  end

end

